require 'spec_helper'

describe StochasticProcess::Base do

  it 'initalizes successfully' do
    s = StochasticProcess::Base.new
    s.should_not be_nil
  end

  it 'has a default time interval and iteration settings' do
    StochasticProcess::Base::DEFAULT_START_TIME.should_not be_nil
    StochasticProcess::Base::DEFAULT_END_TIME.should_not be_nil
    StochasticProcess::Base::DEFAULT_EVALUATIONS.should_not be_nil
  end

  it 'sets default path increment to 0' do
    s = StochasticProcess::Base.new
    s.default_path_increment.should eq 0
  end

  it 'should calculate the interval length correctly' do
    s = StochasticProcess::Base.new(start_time: -10.0, end_time: 10.0,
      evaluations: 100)
    s.interval_length.should eq 20.0
  end

  it 'should calculate a correct mesh given time interval and number of evaluations' do
    s = StochasticProcess::Base.new(start_time: -10.0, end_time: 10.0,
      evaluations: 100)
    s.mesh.should eq 20.0 / 100
  end

  it 'should not allow a non-positive mesh' do
    s = StochasticProcess::Base.new
    expect {s.mesh = -0.1}.to raise_error
  end

  it 'should calculate the correct number of evaluations if a mesh is specified' do
    s = StochasticProcess::Base.new(start_time: -10.0, end_time: 10.0,
      evaluations: 100)
    s.mesh = 0.1
    s.evaluations.should eq 200
  end

  it 'should return a graph with the correct number of elements' do
    s = StochasticProcess::Base.new(start_time: 0.0, end_time: 10.0,
      evaluations: 100)
    s.mesh.should eq 0.1
    s.graph.count.should eq 101 # including starting point
  end

  it 'should return a graph with y coordinates constant to zero' do
    s = StochasticProcess::Base.new(start_time: 0.0, end_time: 10.0,
      evaluations: 100)
    yc = s.graph.collect { |p| p[:y]}
    sumy = yc.inject(0) {|y, res| res + y}
    sumy.should eq 0
  end

end